(in-package :extensions)

(ext:without-package-locks

;;; Added by Chun Tian (binghe)
(defun bind-inet-socket (socket host port)
  "bind a socket to (local) host/port"
  (with-alien ((sockaddr inet-sockaddr))
    (setf (slot sockaddr 'family) af-inet)
    (setf (slot sockaddr 'port) (htons port))
    (setf (slot sockaddr 'addr) (htonl host))
    (when (minusp (unix:unix-bind socket
				  (alien-sap sockaddr)
				  (alien-size inet-sockaddr :bytes)))
      (let ((errno (unix:unix-errno)))
	(unix:unix-close socket)
	(error 'socket-error
	       :format-control "Error binding socket to port ~A: ~A"
	       :format-arguments (list port
				       (unix:get-unix-error-msg))
	       :errno errno)))))

(defun connect-to-inet-socket (host port &optional (kind :stream)
			       &key local-host local-port)
  "The host may be an address string or an IP address in host order."
  (let* ((addr (if (stringp host)
		   (host-entry-addr (or (lookup-host-entry host)
					(error "Unknown host: ~S." host)))
		   host))
	 (socket (create-inet-socket kind)))
    ;; Added by Chun Tian (binghe)
    (when (and local-host local-port)
      (bind-inet-socket socket local-host local-port))
    (with-alien ((sockaddr inet-sockaddr))
      (setf (slot sockaddr 'family) af-inet)
      (setf (slot sockaddr 'port) (htons port))
      (setf (slot sockaddr 'addr) (htonl addr))
      (when (minusp (unix:unix-connect socket
				       (alien-sap sockaddr)
				       (alien-size inet-sockaddr :bytes)))
        ;; unix-close may affect errno; save the current value(s)
        (let ((errno (unix:unix-errno))
              (errmsg (unix:get-unix-error-msg)))
          (unix:unix-close socket)
          (error 'socket-error
                 :format-control "Error connecting socket to [~A:~A]: ~A"
                 :format-arguments (list (if (stringp host)
                                             host
					     (ip-string addr))
                                         port
                                         errmsg)
                 :errno errno)))
      socket)))

(defun create-inet-socket (&optional (kind :stream) &key local-host local-port)
  (multiple-value-bind (proto type)
		       (internet-protocol kind)
    (let ((socket (unix:unix-socket af-inet type proto)))
      (when (minusp socket)
	(error 'socket-error
	       :format-control"Error creating socket: ~A"
	       :format-arguments (list (unix:get-unix-error-msg))
	       :errno (unix:unix-errno)))      
      ;; Added by Chun Tian (binghe)
      (when (and local-host local-port)
	(bind-inet-socket socket local-host local-port))
      socket)))

) ; ext:without-package-locks
